#!/bin/bash
diter=""		# single iteration per sample
diter="-DdITER"	# uncomment for double iteration per sample


accumsize=$(( $1 ))
if [ $accumsize -lt 8 ] || [ $accumsize -gt 32 ]; then
	printf "\n Usage: $0 <algorithm-word-size> <algorithm>\n Word-size range is 8-32\n"
	exit
fi

if [ ! -f "src/algorithmlist.cfg" ]; then
	echo Aborted!  src/algorithmlist.cfg missing
	echo
	exit
fi
read eng engname hashname line <<<$( grep -Fw "$2" "src/algorithmlist.cfg" )
if [ "$eng" != "$2" ]; then
	printf "\n Usage: $0 <algorithm-word-size> <algorithm>\n Specify PRNG algorithm from first column of list:\n"
	cat "src/algorithmlist.cfg"
	exit
fi
enginetitle="${engname}$(($accumsize*2))(${accumsize})${hashname}"
enginedir="tests/engine-${eng}-s${accumsize}"
candidateslist="${enginedir}/candidates-${eng}-s${accumsize}${diter:2:1}.cfg"
outfile="${enginedir}/PRgridscores-${eng}-s${accumsize}${diter:2:1}"

if [ -z $diter ]; then # single iteration per sample
	testingdir="${enginedir}/pract"
	string3="single"
	top=$(( $accumsize - 1 ))
	stepsize=1
else # double iteration per sample
	testingdir="${enginedir}/pract-diter"
	string3="double"
	top=$(( $accumsize * 2 - 1 ))
	stepsize=2
fi
if [ ! -d "$testingdir" ]; then
	echo Aborted!  Practrand scoring of $enginetitle not yet run
	echo
	exit
fi
if [ ! -f "$candidateslist" ]; then
	echo Aborted!  $enginetitle candidate set, $candidateslist, missing
	echo
	exit
fi

dfperiod=$(( 2**($accumsize*2-10) * 2 ))  # in KB
if [ $dfperiod -ge $((1024**5)) ]; then
	dfperiod=$(($dfperiod / 1024**5))" EB"

elif [ $dfperiod -ge $((1024**4)) ]; then
	dfperiod=$(($dfperiod / 1024**4))" PB"

elif [ $dfperiod -ge $((1024**3)) ]; then
	dfperiod=$(($dfperiod / 1024**3))" TB"

elif [ $dfperiod -ge $((1024**2)) ]; then
	dfperiod=$(($dfperiod / 1024**2))" GB"

elif [ $dfperiod -ge 1024 ]; then
	dfperiod=$(($dfperiod / 1024))" MB"
else
	dfperiod=${dfperiod}" KB"
fi
declare -A EXPMAP
EXPMAP[' ']='0'
EXPMAP['K']='10'
EXPMAP['M']='20'
EXPMAP['G']='30'
EXPMAP['T']='40'
EXPMAP['P']='50'
EXPMAP['E']='60'



function  extract_score {
	score="    0K"
	if [ -f "$PRreport" ]; then
		if (grep -Fq "FAIL" "$PRreport"); then
		score=$( grep -F "length=" "$PRreport" |tail -n1 |awk '{ printf "  %3s%1s", $2, toupper(substr($3,1,1)) }' )
		fi
	else
		echo Aborted!  Scoring file, $PRreport missing
		exit
	fi
	printf "$score"
	exponent=$( awk '{ printf "%d", log( $1 ) / log( 2 ) }' <<<${score:2:3} )
	exponent=$(( $exponent + ${EXPMAP[${score:5:1}]} ))
#	printf "   $exponent "
}


# Done as a function to allow a single opened tee command (Much faster running!)
function  print_table {
	printf "\n__________________________________________________________________________________________________________\n  Gridded scores of $string3 iterated $enginetitle candidate [$a $b $c $d].\n      Byte Sampled Double Full Period = ${dfperiod}\n      ${PRoptions}\n    |="
# Word aperture labelling
#	if [ $accumsize -gt 9 ]; then  # Duplicate tests and filenames
#		printf " %2d:00 %2d:00 %2d:00 %2d:01 %2d:02" $top $(($top-1)) $(($top-2)) $top $top
#	fi
# Labelling of aperture positions
	for pos in `seq 0 $top`
	do
		printf "==%02d==" $pos
	done
	printf "\n"

	reportsuffix="${diter:2:1}-a${a}b${b}c${c}d${d}.out"
# Fill out each score case
	totale=0; totald=0
	smalle=$(( $accumsize * 2 + 2 )); smalld=0
	sampsize=$(( $top + 1 ))
#	while [ $sampsize -gt 0 ]
	while [ $sampsize -ge 4 ]
	do
		printf " %02d |" $sampsize
		pos=0
		while [ $pos -le $top ]
		do
			PRreport="${scoredir}/PRscore-samp${sampsize}+${pos}${reportsuffix}"

			extract_score

			totale=$(( $totale + $exponent )); totald=$(( $totald + 1 )) # tele it up
			if [ $exponent -lt $smalle ]; then # min()
#				smalle=$(( $exponent < $smalle ? $exponent : $smalle ))  # min()
				smalle=$exponent
				smalld=1
			elif [ $exponent -eq $smalle ]; then
				smalld=$(( $smalld + 1 ))
			fi
			pos=$(($pos + 1))
		done

#		if [ $sampsize -gt 16 ]; then
#			sampsize=$(( $sampsize - ($stepsize * 2) ))
#		else
#			sampsize=$(( $sampsize - $stepsize ))
#		fi
		sampsize=$(( $sampsize / 2 ))
		if [ $sampsize -gt 4 ] && [ $(( $sampsize / 8 * 8 )) -ne $sampsize ]; then
			sampsize=$(( ($sampsize / 8 + 1) * 8 ))  # round up to multiple of 8
		fi
		printf "\n"
	done

	meane=$(( $totale * 1000 / $totald ))

#	weighe=$(( $meane + $smalle * 1000 - 20000 ))
	weighe=$(( $meane - ($accumsize * 2 - $smalle) * 100 ))
#	weighe=$(( ($totale + $smalle * $smalld) * 1000 / $accumsize**2 ))
#	weighe=$(( ($totale + $accumsize*2 - $smalle * $smalld / $accumsize) * 1000 / $accumsize**2 ))
	printf "  Exponent Minimum = $smalle   Exponent Average = $(($meane/1000)).%03d\n\n" $(($meane%1000))
#	printf " $smalle $smalld $meane $weighe  Exponent Average = $(($meane/1000)).%03d   Weighted = $(($weighe/1000)).%03d\n\n" $(($meane%1000)) $(($weighe%1000))

	printf "[$a $b $c $c],  $smalle,  $(($meane/1000)).%03d\n" $(($meane%1000)) >>"${outfile}.csv"
}



printf "\n\n__________________________________________________________________________________________________________\n  Assembled at $(date +"%F %T %z")\n"  |tee -a "${outfile}.txt"

PRoptions=$(grep -F "options:" "${enginedir}/PRscore-${eng}-s${accumsize}.log" |tail -n1)

printf "\nScoring summary for $string3 iterated sampling of ${enginetitle}\n\nCandidate, Exponent Minimum, Exponent Average\n" >>"${outfile}.csv"

while IFS=", " read a b c d line
do
	a=$(($a));b=$(($b));c=$(($c));d=$(($d))
#	a=13;b=5;c=10;d=9
	if [ $a -ne 0 ] && [ $b -ne 0 ] && [ $c -ne 0 ]; then
		scoredir="${testingdir}/a${a}b${b}c${c}"
		if [ -d "$scoredir" ]; then
			print_table  |tee -a "${outfile}.txt"
		else
			echo Aborted!  Practrand scoring directory, $scoredir, missing
			echo
			exit
		fi
	fi
done < "$candidateslist"

