#!/bin/bash
diter=""		# single iteration per sample
diter="-DdITER"	# uncomment for double iteration per sample


accumsize=$(( $1 ))
if [ $accumsize -lt 8 ] || [ $accumsize -gt 32 ]; then
	printf "\n Usage: $0 <algorithm-word-size> <algorithm>\n Word-size range is 8-32\n"
	exit
fi

if [ ! -f "src/algorithmlist.cfg" ]; then
	echo Aborted!  src/algorithmlist.cfg missing
	echo
	exit
fi
read eng engname hashname line <<<$( grep -Fw "$2" "src/algorithmlist.cfg" )
if [ "$eng" != "$2" ]; then
	printf "\n Usage: $0 <algorithm-word-size> <algorithm>\n Specify PRNG algorithm from first column of list:\n"
	cat "src/algorithmlist.cfg"
	exit
fi
enginetitle="${engname}$(($accumsize*2))(${accumsize})${hashname}"
enginedir="tests/engine-${eng}-s${accumsize}"
testset="${enginedir}/maxiter-${eng}-s${accumsize}.out"

if [ -z $diter ]; then # single iteration per sample
	srcdir="${enginedir}/freq-dist"
	dstdir="${enginedir}/dist-${eng}-s${accumsize}"
	string3="single"
	sampsize=$accumsize
else # double iteration per sample
	srcdir="${enginedir}/freq-dist-diter"
	dstdir="${enginedir}/dist-${eng}-s${accumsize}d"
	string3="double"
	sampsize=$(( $accumsize * 2 ))
fi
if [ ! -f "$testset" ]; then
	echo Aborted!  $enginetitle test set, $testset, missing
	echo
	exit
fi
if [ ! -d "$srcdir" ]; then
	echo Aborted!  Frequency distribution processing for $enginetitle not yet run
	echo
	exit
fi



function  emit_csv {
	head -1 "$1" |tr "\n" " "
	awk '{printf("%s\t",$2)}' "$1"
	tail -2 "$1" |head -1 |awk '{printf("%s\n",$3)}'
}


function  candidate_cases {
	printf "\n    BEGIN  ... at "
	date +"%F %T"

	if [ ! -d "$dstdir" ]; then
		mkdir "$dstdir"
	else
		rm "$dstdir"/*
		rm "${enginedir}/pdist-${eng}-s${accumsize}${diter:2:1}.csv"
		rm "${enginedir}/zdist-${eng}-s${accumsize}${diter:2:1}.csv"
		rm "${enginedir}/nzdist-${eng}-s${accumsize}${diter:2:1}.csv"
	fi

	printf "Merging distribution reports for $string3 iterated $enginetitle random generator.\n"

	pwdir=$( pwd )
	cd "$srcdir"
	names=$( ls -U1 pdist*d0w*.txt )   # list of completed frequency distribution tests
	cd "$pwdir"

	while IFS="-" read -r line line comb line   #  one name at a time, strip down to constants part of the name
	do
		IFS="abcd" read -r line a b c line <<<"$comb"   # extract the individual constants
		a=$(($a)); b=$(($b)); c=$(($c))   # converts from strings to integers
		printf "a${a}b${b}c${c}\n"
		shortname=$( printf "%02d%02d%02d" $a $b $c )
		d=0
		while [ $d -lt $accumsize ]
		do
			srcname="-s${accumsize}-a${a}b${b}c${c}d${d}w${sampsize}p0"
			if [ ! -f "${srcdir}/pdist${srcname}.txt" ]; then
				break;
			fi
			cat "${srcdir}/pdist${srcname}.bin" >>"${dstdir}/${shortname}.pb"
			cat "${srcdir}/zdist${srcname}.bin" >>"${dstdir}/${shortname}.zb"
			cat "${srcdir}/nzdist${srcname}.bin" >>"${dstdir}/${shortname}.nzb"
			cat "${srcdir}/pdist${srcname}.txt" >>"${dstdir}/pdist-s${accumsize}-a${a}b${b}c${c}.txt"
			cat "${srcdir}/zdist${srcname}.txt" >>"${dstdir}/zdist-s${accumsize}-a${a}b${b}c${c}.txt"
			cat "${srcdir}/nzdist${srcname}.txt" >>"${dstdir}/nzdist-s${accumsize}-a${a}b${b}c${c}.txt"
			emit_csv "${srcdir}/pdist${srcname}.txt" >>"${enginedir}/pdist-${eng}-s${accumsize}${diter:2:1}.csv"
			emit_csv "${srcdir}/zdist${srcname}.txt" >>"${enginedir}/zdist-${eng}-s${accumsize}${diter:2:1}.csv"
			emit_csv "${srcdir}/nzdist${srcname}.txt" >>"${enginedir}/nzdist-${eng}-s${accumsize}${diter:2:1}.csv"

			if [ "$eng" == "xoo" ] || [ "$eng" == "xoop" ]; then
				d=$accumsize
			else
				d=$(($d + 1))
			fi
		done
	done <<<"$names"

	printf "Merge completed at "; date +"%F %T"
}


candidate_cases |tee -a "${enginedir}/dist-${eng}-s${accumsize}.log"

