#!/bin/bash
diter=""		# single iteration per sample
diter="-DdITER"	# uncomment for double iteration per sample


accumsize=$(( $1 ))
if [ $accumsize -lt 8 ] || [ $accumsize -gt 32 ]; then
	printf "\n Usage: $0 <algorithm-word-size> <algorithm>\n Word-size range is 8-32\n"
	exit
fi

if [ ! -f "src/algorithmlist.cfg" ]; then
	echo Aborted!  src/algorithmlist.cfg missing
	echo
	exit
fi
read eng engname hashname line <<<$( grep -Fw "$2" "src/algorithmlist.cfg" )
if [ "$eng" != "$2" ]; then
	printf "\n Usage: $0 <algorithm-word-size> <algorithm>\n Specify PRNG algorithm from first column of list:\n"
	cat "src/algorithmlist.cfg"
	exit
fi
enginetitle="${engname}$(($accumsize*2))(${accumsize})${hashname}"
enginedir="tests/engine-${eng}-s${accumsize}"
testset="${enginedir}/maxiter-${eng}-s${accumsize}.out"

if [ -z $diter ]; then # single iteration per sample
	testdir="${enginedir}/freq-dist"
	string3="Single"
	sampsize=$accumsize
else # double iteration per sample
	testdir="${enginedir}/freq-dist-diter"
	string3="Double"
	sampsize=$(( $accumsize * 2 ))
fi
if [ ! -d "tests" ]; then
	mkdir "tests"
fi
if [ ! -d "$enginedir" ]; then
	mkdir "$enginedir"
fi
if [ ! -d "$testdir" ]; then
	mkdir "$testdir"
fi
if [ ! -f "$testset" ]; then
	echo Aborted!  $enginetitle test set, $testset, missing
	echo
	exit
fi
#enoughram=$((5 * 2**20))   # amount of spare ram, in KB, required before running next test case
enoughram=$(( 2**($accumsize*2-10) + 2**20 ))   # 16*2-10=22 => 4G+1G=5G  (-10 for 8-bit elements, -9 for 16-bit elements)



function  cpu_wait {
	# Check for spare MIPS and spare RAM
	while true
	do
		sleep 0.5

#		spareram=$( awk '/MemAvailable/{print $2}' /proc/meminfo )
		read cpu spareram line <<<"$( grep -F "MemAvailable:" /proc/meminfo )"

		read cpu user nice system idle iowait irq softirq steal line <"/proc/stat"
		activeafter=$(( $user + $nice + $system + $softirq + $steal ))
		totalafter=$(( $activeafter + $idle + $iowait ))

		currload=$(( 101 * ($activeafter - $activebefore) / ($totalafter - $totalbefore) ))
		activebefore=$activeafter
		totalbefore=$totalafter

		cpuload=$(( $currload > $prevload ? $currload : $prevload ))  # max()
		prevload=$currload

		if [ $cpuload -lt 48 ] && [ $spareram -gt $enoughram ]; then  # Suited to dual threaded processor cores
			break
		fi
#		printf "cpuload=${currload}  \r" >&2

		sleep 0.5
	done
}


function  test_case {
	case "$eng" in
	xon)
		m=$((-1))
		e="$( "${enginedir}/xon-corr" $a $b $c $d $m )"
		string4=", correction ${e}:${m}"
		;;
	xofb)
		m=$((0x46F5))
		e="$( "${enginedir}/xon-corr" $a $b $c $d $m )"
		string4=", correction ${e}:${m}"
		;;
	*)
		m=$((0))
		e=$((0))
		string4=""
		;;
	esac

	(printf "Frequency distribution data for candidate [${a} ${b} ${c} ${d}]${string4}, begin "; date +"%F %T")

	RNGbin="case-s${accumsize}-a${a}b${b}c${c}d${d}"

	# Compile candidate test case
	gcc -o "${testdir}/prngdist.o" -c "src/${engname,,}128${hashname,,}.c" -O3 -DACCUM_SIZE=$accumsize -DCONSTANT_A=$a -DCONSTANT_B=$b -DCONSTANT_C=$c -DCONSTANT_D=$d -DCONSTANT_E=$e -DCONSTANT_M=$m

	if [ $? -ne 0 ]; then
		echo $enginetitle test case aborted!  Compile error
		echo
		exit 2
	fi

	gcc -o "${testdir}/${RNGbin}" "${testdir}/prngdist.o" "src/freq-dist.c" -O3 -DACCUM_SIZE=$accumsize -DCONSTANT_A=$a -DCONSTANT_B=$b -DCONSTANT_C=$c -DCONSTANT_D=$d -DCONSTANT_E=$e -DCONSTANT_M=$m -DSAMPLE_SIZE=$sampsize -DSAMPLE_POSITION=0 -DENGINETITLE=\"${enginetitle}\" $diter

	if [ $? -ne 0 ]; then
		echo $enginetitle test case aborted!  Compile error
		echo
		exit 1
	fi

	rm "${testdir}/prngdist.o"

	# Run test cases, concurrently
	pwdir=$( pwd )
	cd "$testdir"
	nice -10 stdbuf -o L -e L "./${RNGbin}" &
	cd "$pwdir"

	cpu_wait
	rm "${testdir}/${RNGbin}"	# Delete the binary, even though tests are still running
}



function  candidate_cases {
	printf "\n\n__________________________________________________________________________________________________________\n    BEGIN  ... at "
	date +"%F %T"
	printf "$string3 iterated $enginetitle pseudo-random number generator\n\n"

	read cpu user nice system idle iowait irq softirq steal guest <"/proc/stat"
	activebefore=$(( $user + $nice + $system + $softirq + $steal ))
	totalbefore=$(( $activeafter + $idle + $iowait ))
	prevload=0

	# Compile Xoroshiron correction algorithm
	gcc -o "${enginedir}/xon-corr" "src/xon-correction.c" -O3 -DACCUM_SIZE=$accumsize # -DSTATUS_ON

	while IFS=", " read a b c line
	do
#		read -a e <<< "$line"   # use when file contains pre-built "e" correction table.  Array use: ${e[$d]}
		d=0
		while [ $d -lt $accumsize ]
		do
			if [ -f "${testdir}/pdist-s${accumsize}-a${a}b${b}c${c}d${d}w${sampsize}p0.txt" ]; then
				printf "[${a} ${b} ${c} ${d}] distribution data already present, skipping\n"
			else
				test_case
			fi
			if [ "$eng" == "xoo" ] || [ "$eng" == "xoop" ]; then
				d=$accumsize
			else
				d=$(($d + 1))
			fi
		done
	done < "$testset"

	rm "${enginedir}/xon-corr"
}


candidate_cases |tee -a "${enginedir}/dist-${eng}-s${accumsize}.log"

