#include <limits.h>
#include <stdlib.h>
#include <stdio.h>
#include <stdint.h>
#include <string.h>
#include <time.h>
#include <math.h>


//#define  ACCUM_SIZE  16       // Passed in as -D gcc parameter
//#define  CONSTANT_A  6        // Passed in as -D gcc parameter
//#define  CONSTANT_B  2        // Passed in as -D gcc parameter
//#define  CONSTANT_C  5        // Passed in as -D gcc parameter
//#define  CONSTANT_D  5        // Passed in as -D gcc parameter
//#define  pARITY               // Passed in as -D gcc parameter
//#define  SAMPLE_SIZE     16   // Passed in as -D gcc parameter


#ifdef dITER
	#define  PAIRPOW  (1UL<<SAMPLE_SIZE)
#else
	#define  PAIRPOW  (1UL<<(SAMPLE_SIZE*2))
#endif

#include "xorotypes.h"


#define STR_HELPER(x) #x
#define STR(x) STR_HELPER(x)



int  main(int argc, char* argv[])
{
	uint8_t  *pairs;
	uint32_t  pfreq[256];
	uint32_t  zfreq[256], nzfreq[256], freqi, ztally, nztally, next_prn, prev_prn;
#if ACCUM_SIZE > 16
	uint64_t  pairi;
#else
	uint32_t  pairi;
#endif
	uint64_t  total;
	FILE  *fh;
	uint8_t   fn[64];
	struct timespec  start, stop;
	double  duration;


	// Debugging the PRNG
//	for( freqi = 0; freqi < 80; freqi++ )
//		printf( "%08x\n", nextrnd() );
//	printf( "\n" );
//	exit(0);

	// initialize pair tally array
	pairs = (uint8_t *)calloc( PAIRPOW, sizeof(uint8_t) );    // Zero'd memory allocation - big block

	if( pairs == NULL )
	{
		printf( "malloc error\n" );
		exit(1);
	}

	clock_gettime( CLOCK_MONOTONIC, &start );
	printf( "Started at %.3f   Malloc size = 0x%lx\n", start.tv_sec + start.tv_nsec/1.0e9, PAIRPOW );


#if !defined( dITER )
	// xoroshiro, single iterated, initialize prev_prn for building the pairs
	prev_prn = rotr( nextrnd(), SAMPLE_POSITION ) & SAMPLE_MASK;
#endif

	// iterate full period of the PRNG and tally its output pair occurances
	//  - random ordered 8-bit memory writes
	pairi = PAIRPOW-1;
	do {
	#ifdef dITER
		next_prn = rotr( (drnword_t)nextrnd() | ((drnword_t)nextrnd() << ACCUM_SIZE), SAMPLE_POSITION ) & SAMPLE_MASK;
		pairs[next_prn]++;
	#else
		next_prn = rotr( nextrnd(), SAMPLE_POSITION ) & SAMPLE_MASK;
		pairs[ prev_prn | (next_prn << SAMPLE_SIZE) ]++;  // 8-bit limited tallies
		prev_prn = next_prn;
	#endif
	} while( --pairi );




	clock_gettime( CLOCK_MONOTONIC, &stop );
	duration = (time_t)(stop.tv_sec - start.tv_sec) + (long)(stop.tv_nsec - start.tv_nsec)/1.0e9;
	printf( ENGINETITLE" [%d %d %d %d] paired at %.3f  -  duration = %.3f\n",
		CONSTANT_A, CONSTANT_B, CONSTANT_C, CONSTANT_D, stop.tv_sec + stop.tv_nsec/1.0e9, duration );


	// initialize freq arrays
	memset( pfreq, 0, sizeof(pfreq) );
	memset( zfreq, 0, sizeof(zfreq) );
	memset( nzfreq, 0, sizeof(nzfreq) );

	// read pair array sequentially forward (fast) and increment freq values (fits in L1 caches easy)
	pairi = 0;  ztally = 0;  nztally = 0;
	do {
		// pair freq values
		freqi = pairs[pairi++];
		pfreq[freqi]++;         // 8-bit limited indexing

		// zero-run freq values
		if( freqi == 0 )  ztally++;
		else if( ztally != 0 )
		{
			if( ztally > 255 )  ztally = 255;
			zfreq[ztally]++;
			zfreq[0]++;
			ztally = 0;
		}

		// non-zero-run freq values
		if( freqi != 0 )  nztally++;
		else if( nztally != 0 )
		{
			if( nztally > 255 )  nztally = 255;
			nzfreq[nztally]++;
			nzfreq[0]++;
			nztally = 0;
		}
	} while( pairi < PAIRPOW-1 );

	// pair freq values, tail record
	freqi = pairs[pairi];
	pfreq[freqi]++;

	// zero-run freq values, tail record
	if( freqi == 0 )  ztally++;

	if( ztally != 0 )
	{
		if( ztally > 255 )  ztally = 255;
		zfreq[ztally]++;
		zfreq[0]++;
	}

	// non-zero-run freq values, tail record
	if( freqi != 0 )  nztally++;

	if( nztally != 0 )
	{
		if( nztally > 255 )  nztally = 255;
		nzfreq[nztally]++;
		nzfreq[0]++;
	}


	clock_gettime( CLOCK_MONOTONIC, &stop );
	duration = (time_t)(stop.tv_sec - start.tv_sec) + (long)(stop.tv_nsec - start.tv_nsec)/1.0e9;
	printf( ENGINETITLE" [%d %d %d %d] finished at %.3f  -  total duration = %.3f\n",
		CONSTANT_A, CONSTANT_B, CONSTANT_C, CONSTANT_D, stop.tv_sec + stop.tv_nsec/1.0e9, duration );

	#define NAMESTR  "-s%d-a%db%dc%dd%dw%dp%d"
	#define DESCSTR  ENGINETITLE" [%d %d %d %d], sampling aperture %d>>%d, compensation %x:%x\n", CONSTANT_A, CONSTANT_B, CONSTANT_C, CONSTANT_D, SAMPLE_SIZE, SAMPLE_POSITION, CONSTANT_E, CONSTANT_M

	// write freq array to file - six files, three binary and three text
	// first binary file
	snprintf( fn, sizeof(fn), "pdist"NAMESTR".bin", ACCUM_SIZE, CONSTANT_A, CONSTANT_B, CONSTANT_C, CONSTANT_D, SAMPLE_SIZE, SAMPLE_POSITION );
	fh = fopen( fn, "w" );
	if( fh != NULL )
	{
		fwrite( pfreq, sizeof(pfreq), 1, fh );
		fclose( fh );
	}
	else	printf( "pbin file error\n" );


	// second binary file
	snprintf( fn, sizeof(fn), "zdist"NAMESTR".bin", ACCUM_SIZE, CONSTANT_A, CONSTANT_B, CONSTANT_C, CONSTANT_D, SAMPLE_SIZE, SAMPLE_POSITION );
	fh = fopen( fn, "w" );
	if( fh != NULL )
	{
		fwrite( zfreq, sizeof(zfreq), 1, fh );
		fclose( fh );
	}
	else	printf( "zbin file error\n" );


	// third binary file
	snprintf( fn, sizeof(fn), "nzdist"NAMESTR".bin", ACCUM_SIZE, CONSTANT_A, CONSTANT_B, CONSTANT_C, CONSTANT_D, SAMPLE_SIZE, SAMPLE_POSITION );
	fh = fopen( fn, "w" );
	if( fh != NULL )
	{
		fwrite( nzfreq, sizeof(nzfreq), 1, fh );
		fclose( fh );
	}
	else	printf( "nzbin file error\n" );


	// first text file
	snprintf( fn, sizeof(fn), "pdist"NAMESTR".txt", ACCUM_SIZE, CONSTANT_A, CONSTANT_B, CONSTANT_C, CONSTANT_D, SAMPLE_SIZE, SAMPLE_POSITION );
	fh = fopen( fn, "w" );
	if( fh != NULL )
	{
#ifdef dITER
		fprintf( fh, "Frequency distribution for direct output of "DESCSTR );
#else
		fprintf( fh, "Frequency distribution for pairs of "DESCSTR );
#endif

		total = 0;
		for( freqi = 0; freqi < 256; freqi++ )
		{
			total += pfreq[freqi];
			fprintf( fh, "%3d  %u\n", freqi, pfreq[freqi] );
		}

		fprintf( fh, "Total = %lu or 0x%lx\n\n", total, total );
		fclose( fh );
	}
	else	printf( "ptxt file error\n" );


	// second text file
	snprintf( fn, sizeof(fn), "zdist"NAMESTR".txt", ACCUM_SIZE, CONSTANT_A, CONSTANT_B, CONSTANT_C, CONSTANT_D, SAMPLE_SIZE, SAMPLE_POSITION );
	fh = fopen( fn, "w" );
	if( fh != NULL )
	{
		fprintf( fh, "Frequency distribution for zero runs of "DESCSTR );

		total = 0;
		for( freqi = 0; freqi < 256; freqi++ )
		{
			total += zfreq[freqi];
			fprintf( fh, "%3d  %u\n", freqi, zfreq[freqi] );
		}

		fprintf( fh, "Total = %lu or 0x%lx\n\n", total, total );
		fclose( fh );
	}
	else	printf( "ztxt file error\n" );


	// third text file
	snprintf( fn, sizeof(fn), "nzdist"NAMESTR".txt", ACCUM_SIZE, CONSTANT_A, CONSTANT_B, CONSTANT_C, CONSTANT_D, SAMPLE_SIZE, SAMPLE_POSITION );
	fh = fopen( fn, "w" );
	if( fh != NULL )
	{
		fprintf( fh, "Frequency distribution for non-zero runs of "DESCSTR );

		total = 0;
		for( freqi = 0; freqi < 256; freqi++ )
		{
			total += nzfreq[freqi];
			fprintf( fh, "%3d  %u\n", freqi, nzfreq[freqi] );
		}

		fprintf( fh, "Total = %lu or 0x%lx\n\n", total, total );
		fclose( fh );
	}
	else	printf( "nztxt file error\n" );


	free( pairs );
	return 0;
}
