#!/bin/bash
RNGbin=$1
PRcommand=$2
PRreport=$3
#
#  This script is intended for use with single case scripting
#


declare -A SCALEMAP
SCALEMAP[' ']='0'
SCALEMAP['K']='1'
SCALEMAP['M']='1024'
SCALEMAP['G']='1024**2'
SCALEMAP['T']='1024**3'
SCALEMAP['P']='1024**4'
SCALEMAP['E']='1024**5'



function extract_score {
	scoreline=$( grep -F "FAIL" "$PRreport" )
	if [ "$scoreline" == "" ]; then   # empty sring in quotes counts as one line
		failcnt=0
	else
		failcnt=$( wc -l <<<"$scoreline" )
	fi
	bcfncnt=$( grep -F "BCFN" <<<"$scoreline" |wc -l )  # BCFN tests have small glitches for some reason
	scoresize=$( grep -F "length=" "$PRreport" |tail -n 1 |awk '{ printf "%3s %1s", $2, toupper(substr($3,1,1)) }' )
	sizekb=$(( ${scoresize:0:3} * ${SCALEMAP[${scoresize:4:1}]} ))
}


#
#  Starts here
#
if [ -f "$PRreport" ]; then # pre-existing Practrand report - check for completeness

	extract_score
	if [ $failcnt -eq 0 ]; then # not complete - rerun and score the case
		printf "${RNGbin}, PractRand report incomplete - Retesting ...\n"
		(printf "\nGridding report of ${RNGbin} at "; date +"%F %T"; printf "\n") >"$PRreport" # Overwrite report

		"$RNGbin" | stdbuf -o L $PRcommand >>"$PRreport"
		if [ $? -ne 0 ]; then
			printf "Aborted $RNGbin - PractRand error\n"
			exit 2
		fi

		extract_score
	fi

else # no pre-existing Practrand report - run and score the case
	(printf "\nGridding report of ${RNGbin} at "; date +"%F %T"; printf "\n") >"$PRreport"  # Create report

	"$RNGbin" | stdbuf -o L $PRcommand >>"$PRreport"
	if [ $? -ne 0 ]; then
		printf "Aborted $RNGbin - PractRand error\n"
		exit 2
	fi

	extract_score
fi
#echo bcfncnt = $bcfncnt

# Output the score to console/logfile.  This is not score extraction for score table.
if [ $bcfncnt -eq $failcnt ] && [ ! -f "${PRreport}1" ]; then  # probably incorrect score due to too sensitive BCFN testing
	if [ $bcfncnt -gt 2 ];then
		printf "Accepted because "

	else # rerun testing from +1 power
		mv "$PRreport" "${PRreport}0"
		printf "BCFNs: $bcfncnt, $RNGbin, PractRand score: ${scoresize}B - Trying larger ...\n"
		scoresizedn=$scoresize
		sizeup=$(( $sizekb * 2 ))
		"$RNGbin" | stdbuf -o L $PRcommand -tlmin "${sizeup}KB" >>"$PRreport"

		extract_score
		if [ $sizekb -eq $sizeup ] && [ $bcfncnt -gt 0 ]; then # recuring fails
			mv "$PRreport" "${PRreport}1"

			if [ $bcfncnt -lt $failcnt ] || [ $bcfncnt -gt 2 ]; then # initial score was valid, revert
				mv "${PRreport}0" "$PRreport"
				printf "BCFNs: $bcfncnt, Fails: $failcnt, Reverted - "
				scoresize=$scoresizedn
				bcfncnt=0

			else # two BCFN glitches in a row!  raise another power, rerun again
				printf "BCFNs: $bcfncnt, Fails: $failcnt, $RNGbin  - Larger again ...\n"
				sizeup=$(( $sizekb * 2 ))
				"$RNGbin" | stdbuf -o L $PRcommand -tlmin "${sizeup}KB" >"$PRreport"

				extract_score
				if [ $sizekb -eq $sizeup ] && [ $bcfncnt -gt 0 ]; then # plain giving up at three tries, revert
					mv "$PRreport" "${PRreport}2"
					mv "${PRreport}0" "$PRreport"
					printf "BCFNs: $bcfncnt, Fails: $failcnt, Reverted - "
					scoresize=$scoresizedn
					bcfncnt=0
				fi
			fi
		else # revised report is normal, BCFN glitch cleared
			if [ $bcfncnt -lt $failcnt ]; then # report the correction
				printf "BCFNs: $bcfncnt, Fails: $failcnt, Accepted - "
			fi
		fi
	fi
	if [ $bcfncnt -eq $failcnt ]; then
		printf "BCFN & Fails: $failcnt, "
	fi
fi

rm "$RNGbin"

scoreline=$( grep -F "PractRand score:" "$PRreport" |wc -l )
if [ $scoreline -eq 0 ]; then
	(printf "PractRand score: ${scoresize}B,  finished at "; date +"%F %T") >>"$PRreport"
fi
printf "$RNGbin, PractRand score: ${scoresize}B\n"
